#include <QEvent>
#include <QApplication>
#include <QTimer>
#include "myevent.h"
#include "spacenav.h"

// using spacenav::SpaceNav;

SpaceNav::SpaceNav() {
    handle = hid_open(0x046d, 0xc626, NULL);
    if (!handle) {
        qDebug("Couldn't open space navigator");
		exit(1);
    }
    // Set the hid_read() function to be non-blocking.
    hid_set_nonblocking(handle, 1);

	timer1 = new QTimer( this );
	connect ( timer1, SIGNAL(timeout()), this, SLOT( sampling() ) );

    memset(buf,0,sizeof(buf));
    result = 0;
}

void SpaceNav::sampling()
{
	#define UMBRAL1 30
	#define UMBRAL2  5
    int tmp, flag=0;
	int vx, vy, vz, t1, t2, t3;

	vx = vy = vz = t1 = t2 = t3 = 0;

    result = hid_read(handle, buf, sizeof(buf));
	action = (int)buf[0];
	
    if( result == 3){//Buttons pressed
		if(buf[1] == LeftButton || buf[1] == RigthButton){
			MyEvent event = MyEvent( );
			event.accion = action;
			event.b = buf[1];
			QApplication::sendEvent( glWidget, &event );
		}
    }
	else if( result == 7){
        if( action == Translation){//Translation 3 degrees of freedom
            //In X axis (left - right)
			// fprintf( stderr, "%x %x\n", buf[2], buf[1] );
			switch ( buf[2] ) {
				case 0x00 :
                	tmp = buf[1];
					if( tmp > UMBRAL1 ) {
						x = tmp;
						flag++; 
					}
					break;
				case 0xff:
                	tmp = buf[1] - NEGATIVE ;
					// fprintf( stderr, "%d\n", tmp );
					if( tmp < -UMBRAL1 ) {
						x = tmp;
						flag++; 
					}
					break;
				case 0xfe:
					x = -400;
					flag++;
					break;
				case 0x1:
					x = 400;
					flag++;
					break;
			}
			
            //In Y axis (down - up)
			switch ( buf[6] ) {
				case 0x00 :
                	tmp = buf[5];
					if( tmp > UMBRAL1 ) {
						y = tmp;
						flag++;
					}
					break;
				case 0xff:
                	tmp = buf[5] - NEGATIVE ;
					if( tmp < -UMBRAL1 ) {
						y = tmp;
						flag++;
					}
					break;
				case 0xfe:
					y = -400;
					flag++;
					break;
				case 0x1:
					y = 400;
					flag++;
					break;
			}

            //In Z axis (back - forward)
			switch ( buf[4] ) {
				case 0x00 :
                	tmp = buf[3];
					if( tmp > UMBRAL1 ) {
						z = tmp;
						flag++;
					}
					break;
				case 0xff:
                	tmp = buf[3] - NEGATIVE ;
					// fprintf( stderr, "%d\n", tmp );
					if( tmp < -UMBRAL1 ) {
						z = tmp;
						flag++;
					}
					break;
				case 0xfe:
					z = -400;
					flag++;
					break;
				case 0x1:
					z = 400;
					break;
			}
			if ( flag ) {
				MyEvent event = MyEvent( );
				event.accion = action;
				event.vx = x - UMBRAL1;
				event.vy = y - UMBRAL1;
				event.vz = z - UMBRAL1;
				QApplication::sendEvent( glWidget, &event);
			}
        }else if( action == Rotation){//Rotation 3 degrees of freedom
            //In X axis (pitch)
			switch ( buf[2] ) {
				case 0x00 :
                	tmp = buf[1];
					if( tmp > UMBRAL2 ) {
						t1 = tmp;
						flag++;
					}
					break;
				case 0xff:
                	tmp = buf[1] - NEGATIVE ;
					if( tmp < -UMBRAL2 ) {
						t1 = tmp;
						flag++;
					}
					break;
				case 0xfe:
					t1 = -400;
					flag++;
					break;
				case 0x1:
					t1 = 400;
					flag++;
					break;
			}

            //In Y axis (yaw)
			switch ( buf[6] ) {
				case 0x00 :
                	tmp = buf[5];
					if( tmp > UMBRAL2 ) {
						t2 = tmp;
						flag++;
					}
					break;
				case 0xff:
                	tmp = buf[5] - NEGATIVE ;
					if( tmp < -UMBRAL2 ) {
						t2 = tmp;
						flag++;
					}
					break;
				case 0xfe:
					t2 = -400;
					flag++;
					break;
				case 0x1:
					t2 = 400;
					flag++;
					break;
			}

            //In Z axis (roll)
			switch ( buf[4] ) {
				case 0x00 :
                	tmp = buf[3];
					if( tmp > UMBRAL2 ) {
						t3 = tmp;
						flag++;
					}
					break;
				case 0xff:
                	tmp = buf[3] - NEGATIVE ;
					if( tmp < -UMBRAL2 ) {
						t3 = tmp;
						flag++;
					}
					break;
				case 0xfe:
					t3 = -400;
					flag++;
					break;
				case 0x1:
					t3 = 400;
					flag++;
					break;
			}
			if ( flag ) {
				MyEvent event = MyEvent( );
				event.accion = action;
				event.tx = t1 - UMBRAL2;
				event.ty = t2 - UMBRAL2;
				event.tz = t3 - UMBRAL2;
				QApplication::sendEvent( glWidget, &event);
			}
        }
    }
}
